(***********************************************************************)
(*                                                                     *)
(*                       Ambients in JoCaml                            *)
(*                                                                     *)
(*               Alan Schmitt, projet Para, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1999 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

(* later we may use a more efficient data structure *)

type ('name, 'id, 'content) t = ('name * 'id * 'content) list ref

let rec list_first p = function
  | a::l -> (match p a with Some v -> v | None -> list_first p l)
  | [] -> raise Not_found 

let rec list_filter p = function
  | a::l when p a -> a::list_filter p l
  | _::l -> list_filter p l
  | [] -> [] 

let create () = ref []

let add r a id v = r := (a,id,v)::!r

let find_id r id' = 
  list_first (function (_,id,v) -> if id = id' then Some v else None) !r 

let find_name r n' = 
  list_first (function (n,id,v) -> if n = n' then Some (id,v) else None) !r 

let find_name_different r n' id' =
  list_first (function (n,id,v) -> if n = n' && not (id = id') then Some (id,v) else None) !r 

let remove r id = 
  r := list_filter (function (_,id',_) -> not (id = id')) !r

let flush r f = 
  let l = !r in r := []; List.iter (function (a,id,v) -> f a id v) l

let to_string r ts = 
  let f (a,id,v) s = ts a id v^" "^s in 
  if !r = [] then "[]" else "[ "^List.fold_right f !r "]"



